@@ -1,6 +1,27 @@
+/* GDM - The GNOME Display Manager
+ * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 #include "config.h"
 
 #include <gtk/gtk.h>
+#include <glib/gi18n.h>
+#include <librsvg/rsvg.h>
+#include "gdmcommon.h"
 #include "gdmwm.h"
 #include "greeter_geometry.h"
 #include "greeter_canvas_item.h"
@@ -146,7 +167,7 @@ greeter_size_allocate_fixed (GreeterItemInfo *fixed,
 
       if (child->item_type != GREETER_ITEM_TYPE_LABEL)
 	fixup_from_anchor (&child_allocation, child->anchor);
-      
+
       greeter_item_size_allocate (child,
 				  &child_allocation,
 				  canvas);
@@ -325,6 +346,10 @@ greeter_size_allocate_box (GreeterItemInfo *box,
 	  if (child->item_type != GREETER_ITEM_TYPE_LABEL)
 	    fixup_from_anchor (&child_allocation, child->anchor);
 	  
+	  if (child_allocation.x + child_allocation.width > allocation->x + allocation->width)
+	      child_allocation.width -= (child_allocation.x + child_allocation.width
+					 - allocation->x - allocation->width);
+
 	  greeter_item_size_allocate (child,
 				      &child_allocation,
 				      canvas);
@@ -463,6 +488,7 @@ greeter_item_size_request (GreeterItemInfo *item,
     case GREETER_ITEM_SIZE_BOX:
       set_width = box_requisition.width;
       break;
+    case GREETER_ITEM_SIZE_SCALE:
     case GREETER_ITEM_SIZE_UNSET:
       break;
     }
@@ -478,6 +504,7 @@ greeter_item_size_request (GreeterItemInfo *item,
     case GREETER_ITEM_SIZE_BOX:
       set_height = box_requisition.height;
       break;
+    case GREETER_ITEM_SIZE_SCALE:
     case GREETER_ITEM_SIZE_UNSET:
       break;
     }
@@ -492,8 +519,7 @@ greeter_item_size_request (GreeterItemInfo *item,
        * You can go about your business.
        * Move Along
        */
-      text = greeter_item_expand_text (item->data.text.orig_text);
-
+      text = gdm_common_expand_text (item->data.text.orig_text);
 
       if (set_width > 0)
 	      max_width = set_width;
@@ -524,6 +550,57 @@ greeter_item_size_request (GreeterItemInfo *item,
       req->width = gdk_pixbuf_get_width (item->data.pixmap.pixbufs[0]);
       req->height = gdk_pixbuf_get_height (item->data.pixmap.pixbufs[0]);
     }
+
+  if (item->item_type == GREETER_ITEM_TYPE_SVG)
+    {
+      GdkPixbuf *svg;
+
+      svg = rsvg_pixbuf_from_file (item->data.pixmap.files[0], NULL);
+      req->width = gdk_pixbuf_get_width (svg);
+      req->height = gdk_pixbuf_get_height (svg);
+      g_object_unref (svg);
+    }
+
+  if (item->item_type == GREETER_ITEM_TYPE_BUTTON)
+    {
+      PangoLayout *layout;
+      PangoFontDescription *font_desc;
+      int pango_width, pango_height;
+      int pix_width, pix_height;
+      
+      GtkWidget *dummy_w = gtk_button_new ();
+
+      if (strcmp (item->id, "options_button") == 0)
+       layout = gtk_widget_create_pango_layout (dummy_w, _("Select Language..."));
+      else
+     layout = gtk_widget_create_pango_layout (dummy_w, item->data.text.orig_text);
+       
+      pango_layout_get_size (layout, &pango_width, &pango_height);
+      
+      pix_height = PANGO_PIXELS (pango_height);
+      pix_width = PANGO_PIXELS (pango_width);
+
+      if (strcmp (item->id, "options_button") == 0)
+     pix_width += pix_height * 2; /* add padding for combobox */
+
+      if (pix_width > item->parent->box_min_width)
+     req->width = pix_width;
+      else
+     req->width = item->parent->box_min_width;
+
+      if (pix_height > item->parent->box_min_height)
+     req->height = pix_height;
+      else
+     req->height = item->parent->box_min_height;
+    }
+ 
+  if (req->width > 0 && req->height > 0)
+    {
+      if (item->width_type == GREETER_ITEM_SIZE_SCALE && set_height > 0)
+        set_width = (req->width * set_height) / req->height;
+      else if (item->height_type == GREETER_ITEM_SIZE_SCALE && set_width > 0)
+        set_height = (req->height * set_width) / req->width;
+    }
   
   if (set_width > 0)
     req->width = set_width;
