@@ -19,11 +19,11 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include <config.h>
+#include "config.h"
+
 #include <gtk/gtk.h>
 
 #include "ve-i18n.h"
-
 #include "ve-misc.h"
 #include "ve-miscui.h"
 
